@@ -3,10 +3,12 @@ package org.gcube.vomanagement.usermanagement.impl;
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Set;
 
 import org.gcube.vomanagement.usermanagement.GroupManager;
 import org.gcube.vomanagement.usermanagement.UserManager;
@@ -16,8 +18,10 @@ import org.gcube.vomanagement.usermanagement.exception.TeamRetrievalFault;
 import org.gcube.vomanagement.usermanagement.exception.UserManagementPortalException;
 import org.gcube.vomanagement.usermanagement.exception.UserManagementSystemException;
 import org.gcube.vomanagement.usermanagement.exception.UserRetrievalFault;
+import org.gcube.vomanagement.usermanagement.exception.VirtualGroupNotExistingException;
 import org.gcube.vomanagement.usermanagement.model.CustomAttributeKeys;
 import org.gcube.vomanagement.usermanagement.model.Email;
+import org.gcube.vomanagement.usermanagement.model.GCubeGroup;
 import org.gcube.vomanagement.usermanagement.model.GCubeMembershipRequest;
 import org.gcube.vomanagement.usermanagement.model.GCubeRole;
 import org.gcube.vomanagement.usermanagement.model.GCubeUser;
@@ -429,11 +433,10 @@ public class LiferayUserManager implements UserManager {
 			//_log.debug("Trying to fetch user by username = " + username);
 			toGet = UserLocalServiceUtil.getUserByScreenName(ManagementUtils.getCompany().getCompanyId(), username);
 			toReturn = mapLRUser(toGet);
-		} catch (PortalException e) {
-			_log.error("Failed to fetch user by username = " + username, e);
-			throw new UserRetrievalFault("User not existing", e);
+		} catch (PortalException e1) {
+			throw new UserRetrievalFault("User not existing", username, e1);
 		} catch (SystemException e) {
-			_log.error("Failed to fetch user by username = " + username, e);
+			_log.error("SystemException: Failed to fetch user by username = " + username);
 			throw new UserManagementSystemException(e.getMessage(), e);
 		}
 		return toReturn;
@@ -458,7 +461,7 @@ public class LiferayUserManager implements UserManager {
 			toGet = UserLocalServiceUtil.getUser(userId);
 			toReturn = mapLRUser(toGet);
 		} catch (PortalException e) {
-			throw new UserRetrievalFault("User not existing", e);
+			throw new UserRetrievalFault("User not existing", userId, e);
 		} catch (SystemException e) {
 			throw new UserManagementSystemException(e.getMessage(), e);
 		}
@@ -476,7 +479,7 @@ public class LiferayUserManager implements UserManager {
 			toGet = UserLocalServiceUtil.getUser(userId);
 			toReturn = toGet.getComments();
 		} catch (PortalException e) {
-			throw new UserRetrievalFault("User not existing", e);
+			throw new UserRetrievalFault("User not existing", userId, e);
 		} catch (SystemException e) {
 			throw new UserManagementSystemException(e.getMessage(), e);
 		}
@@ -493,7 +496,7 @@ public class LiferayUserManager implements UserManager {
 			user.setComments(summary);
 			UserLocalServiceUtil.updateUser(user);
 		} catch (PortalException e) {
-			throw new UserRetrievalFault("User not existing", e);
+			throw new UserRetrievalFault("User not existing", userId, e);
 		} catch (SystemException e) {
 			throw new UserManagementSystemException(e.getMessage(), e);
 		}
@@ -511,7 +514,7 @@ public class LiferayUserManager implements UserManager {
 			toGet = UserLocalServiceUtil.getUserByEmailAddress(ManagementUtils.getCompany().getCompanyId(), emailAddress);
 			toReturn = mapLRUser(toGet);
 		} catch (PortalException e) {
-			throw new UserRetrievalFault("User not existing", e);
+			throw new UserRetrievalFault("emailAddress not existing", e);
 		} catch (SystemException e) {
 			throw new UserManagementSystemException(e.getMessage(), e);
 		}
@@ -532,7 +535,10 @@ public class LiferayUserManager implements UserManager {
 	 * {@inheritDoc}
 	 */
 	@Override
-	public List<GCubeUser> listUsers() throws UserManagementSystemException {
+	public List<GCubeUser> listUsers(boolean indexed) throws UserManagementSystemException {
+		if (indexed)
+			return listUsers();
+		_log.debug("Asked for listUsers non indexed");	
 		List<GCubeUser> toReturn = new ArrayList<GCubeUser>();
 		try {
 			OrderByComparator comparator = OrderByComparatorFactoryUtil.create("User_", "screenname", true);
@@ -552,8 +558,47 @@ public class LiferayUserManager implements UserManager {
 	 * {@inheritDoc}
 	 */
 	@Override
+	public List<GCubeUser> listUsers() throws UserManagementSystemException {
+		List<GCubeUser> toReturn = new ArrayList<GCubeUser>();
+		try {
+			List<User> lrUsers = UserLocalServiceUtil.getUsers(QueryUtil.ALL_POS,  QueryUtil.ALL_POS);
+			for (User user : lrUsers) {
+				if (user.isActive())
+					toReturn.add(mapLRUser(user));
+			}
+		} catch (SystemException e) {
+			throw new UserManagementSystemException(e.getMessage(), e);
+		} catch (PortalException e) {
+			e.printStackTrace();
+		}
+		return toReturn;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
 	public List<GCubeUser> listUsersByGroup(long groupId) throws UserManagementSystemException, GroupRetrievalFault, UserRetrievalFault {
 		List<GCubeUser> toReturn = new ArrayList<GCubeUser>();
+		try {
+			for (User user : UserLocalServiceUtil.getGroupUsers(groupId)) {
+				if (user.isActive())
+					toReturn.add(mapLRUser(user));
+			}
+		} catch (PortalException e) {
+			throw new GroupRetrievalFault("listUsersByGroup not existing group", groupId, e);
+		} catch (SystemException e) {
+			e.printStackTrace();
+		}
+		return toReturn;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public List<GCubeUser> listUsersByGroup(long groupId, boolean indexed) throws UserManagementSystemException, GroupRetrievalFault, UserRetrievalFault {
+		if (indexed)
+			return listUsersByGroup(groupId);
+		List<GCubeUser> toReturn = new ArrayList<GCubeUser>();
 		try {
 			OrderByComparator comparator = OrderByComparatorFactoryUtil.create("User_", "screenname", true);
 			LinkedHashMap<String, Object> params = new LinkedHashMap<String, Object>();
@@ -564,7 +609,7 @@ public class LiferayUserManager implements UserManager {
 					toReturn.add(mapLRUser(user));
 			}
 		} catch (PortalException e) {
-			e.printStackTrace();
+			throw new GroupRetrievalFault("listUsersByGroup not existing group", groupId, e);
 		} catch (SystemException e) {
 			e.printStackTrace();
 		}
@@ -596,11 +641,8 @@ public class LiferayUserManager implements UserManager {
 			Group g = GroupLocalServiceUtil.getGroup(groupId);
 			_log.debug("Asked for users and roles of group: "+g.getName());			
 
-			OrderByComparator comparator = OrderByComparatorFactoryUtil.create("User_", "screenname", true);
-			LinkedHashMap<String, Object> params = new LinkedHashMap<String, Object>();
-			params.put("usersGroups", groupId);			
-			List<User> usersByGroup = UserLocalServiceUtil.search(ManagementUtils.getCompany().getCompanyId(), "", 0, params, QueryUtil.ALL_POS, QueryUtil.ALL_POS, comparator);
-			
+			List<User> usersByGroup = UserLocalServiceUtil.getGroupUsers(groupId);
+
 			for (User user : usersByGroup) {
 				if (user.isActive()) {
 					long userId = user.getUserId();
@@ -615,7 +657,7 @@ public class LiferayUserManager implements UserManager {
 			}			
 
 		} catch (PortalException e1) {
-			throw new GroupRetrievalFault("Group not existing (I think you better check)", e1);
+			throw new GroupRetrievalFault("Group not existing (I think you better check)", groupId, e1);
 		} catch (Exception e) {
 			e.printStackTrace();
 		} 
@@ -668,11 +710,36 @@ public class LiferayUserManager implements UserManager {
 		}
 		return toReturn;
 	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public Set<GCubeUser> getUserContactsByGroup(long userId, long scopeGroupId) throws UserManagementSystemException, GroupRetrievalFault, UserRetrievalFault {
+		Set<GCubeUser> users = new HashSet<>();
+		GroupManager gm = new LiferayGroupManager();
+		try {
+			if (gm.isRootVO(scopeGroupId)) {
+				Set<GCubeGroup> userGroups = gm.listGroupsByUserAndSiteGroupId(userId, scopeGroupId);
+				for (GCubeGroup userGroup : userGroups) {
+					if (gm.isVRE(userGroup.getGroupId())) {
+						users.addAll(listUsersByGroup(userGroup.getGroupId()));
+						_log.debug("getUserContactsByGroup added users of group " + userGroup.getGroupId() + " for userid="+userId);
+					}
+				}
+			} else { //is a VRE
+				users.addAll(listUsersByGroup(scopeGroupId));
+			}
+		} catch (UserManagementSystemException | GroupRetrievalFault | UserRetrievalFault | VirtualGroupNotExistingException e) {
+			e.printStackTrace();
+		}
+		return users;
+	}
 	/*
 	 * END LISTING ENTITIES 
 	 */
 
 
+
 	/*
 	 * MEMBERSHIP REQUESTS
 	 */
@@ -685,7 +752,7 @@ public class LiferayUserManager implements UserManager {
 			Group g = GroupLocalServiceUtil.getGroup(groupId);
 			_log.debug("Asked for pending users list of group: "+g.getName());
 		} catch (PortalException e1) {
-			throw new GroupRetrievalFault("Group not existing", e1);
+			throw new GroupRetrievalFault("Group not existing", groupId, e1);
 		} catch (SystemException e) {
 			e.printStackTrace();
 		} 
@@ -733,7 +800,7 @@ public class LiferayUserManager implements UserManager {
 					MembershipRequestLocalServiceUtil.updateStatus(replyMan.getUserId(), req.getMembershipRequestId(), replyComment, 1, addUserToGroup, new ServiceContext());
 					//the method above just adds the user to the VRE, it is needed to add the user to the parent Sites as well (VO and RootVO)
 					assignUserToGroup(GroupLocalServiceUtil.getGroup(groupId).getParentGroupId(), requestUserId); 
-					
+
 					return mapLRMembershipRequest(MembershipRequestLocalServiceUtil.getMembershipRequest(req.getMembershipRequestId()));
 				}
 			}
@@ -791,7 +858,7 @@ public class LiferayUserManager implements UserManager {
 			Group g = GroupLocalServiceUtil.getGroup(groupId);
 			_log.debug("Asked for pending users list of group: "+g.getName());
 		} catch (PortalException e1) {
-			throw new GroupRetrievalFault("Group not existing", e1);
+			throw new GroupRetrievalFault("Group not existing", groupId, e1);
 		} catch (SystemException e) {
 			e.printStackTrace();
 		} 
@@ -881,7 +948,12 @@ public class LiferayUserManager implements UserManager {
 
 			// date are saved according GMT
 			long creationTime = user.getCreateDate().getTime();
-			long changedTime = user.getPasswordModifiedDate().getTime();
+
+			long changedTime;
+			if(user.getPasswordModifiedDate() != null)
+				changedTime = user.getPasswordModifiedDate().getTime();
+			else
+				return false; // there was no change
 
 			return changedTime  > creationTime;
 		} catch (Exception e) {
@@ -1025,4 +1097,14 @@ public class LiferayUserManager implements UserManager {
 		}
 		return true;
 	}
+	@Override
+	public List<GCubeUser> listUsersByGlobalRole(long roleId) {
+		// TODO Auto-generated method stub
+		return null;
+	}
+	@Override
+	public List<Long> getUserIdsByGroup(long groupId) {
+		// TODO Auto-generated method stub
+		return null;
+	}
 }
